@@ -212,16 +212,16 @@ public class ExternalSort {
          */
         public static long mergeSortedFiles(BufferedWriter fbw,
                 final Comparator<String> cmp, boolean distinct,
-                List<BinaryFileBuffer> buffers) throws IOException {
-                PriorityQueue<BinaryFileBuffer> pq = new PriorityQueue<>(
-                        11, new Comparator<BinaryFileBuffer>() {
+                List<IOStringStack> buffers) throws IOException {
+                PriorityQueue<IOStringStack> pq = new PriorityQueue<>(
+                        11, new Comparator<IOStringStack>() {
                                 @Override
-                                public int compare(BinaryFileBuffer i,
-                                        BinaryFileBuffer j) {
+                                public int compare(IOStringStack i,
+                                IOStringStack j) {
                                         return cmp.compare(i.peek(), j.peek());
                                 }
                         });
-                for (BinaryFileBuffer bfb : buffers) {
+                for (IOStringStack bfb : buffers) {
                         if (!bfb.empty()) {
                                 pq.add(bfb);
                         }
@@ -230,13 +230,13 @@ public class ExternalSort {
                 try {
                         if (!distinct) {
                             while (pq.size() > 0) {
-                                    BinaryFileBuffer bfb = pq.poll();
+                                    IOStringStack bfb = pq.poll();
                                     String r = bfb.pop();
                                     fbw.write(r);
                                     fbw.newLine();
                                     ++rowcounter;
                                     if (bfb.empty()) {
-                                            bfb.fbr.close();
+                                            bfb.close();
                                     } else {
                                             pq.add(bfb); // add it back
                                     }
@@ -244,19 +244,19 @@ public class ExternalSort {
                         } else {
                             String lastLine = null;
                             if(pq.size() > 0) {
-                           BinaryFileBuffer bfb = pq.poll();
+                           IOStringStack bfb = pq.poll();
                            lastLine = bfb.pop();
                            fbw.write(lastLine);
                            fbw.newLine();
                            ++rowcounter;
                            if (bfb.empty()) {
-                             bfb.fbr.close();
+                             bfb.close();
                            } else {
                              pq.add(bfb); // add it back
                            }
                          }
                             while (pq.size() > 0) {
-                          BinaryFileBuffer bfb = pq.poll();
+                        IOStringStack bfb = pq.poll();
                           String r = bfb.pop();
                           // Skip duplicate lines
                           if  (cmp.compare(r, lastLine) != 0) {
@@ -266,7 +266,7 @@ public class ExternalSort {
                           }
                           ++rowcounter;
                           if (bfb.empty()) {
-                            bfb.fbr.close();
+                            bfb.close();
                           } else {
                             pq.add(bfb); // add it back
                           }
@@ -274,7 +274,7 @@ public class ExternalSort {
                         }
                 } finally {
                         fbw.close();
-                        for (BinaryFileBuffer bfb : pq) {
+                        for (IOStringStack bfb : pq) {
                                 bfb.close();
                         }
                 }
@@ -282,6 +282,7 @@ public class ExternalSort {
 
         }
 
+
         /**
          * This merges a bunch of temporary flat files
          *
@@ -392,7 +393,7 @@ public class ExternalSort {
         public static long mergeSortedFiles(List<File> files, File outputfile,
                 final Comparator<String> cmp, Charset cs, boolean distinct,
                 boolean append, boolean usegzip) throws IOException {
-                ArrayList<BinaryFileBuffer> bfbs = new ArrayList<>();
+                ArrayList<IOStringStack> bfbs = new ArrayList<>();
                 for (File f : files) {
                         final int BUFFERSIZE = 2048;
                         InputStream in = new FileInputStream(f);
@@ -419,6 +420,53 @@ public class ExternalSort {
                 return rowcounter;
         }
 
+        /**
+         * This merges a bunch of temporary flat files
+         *
+         * @param files The {@link List} of sorted {@link File}s to be merged.
+         * @param distinct Pass <code>true</code> if duplicate lines should be
+         *                discarded.
+         * @param fbw The output {@link BufferedWriter} to merge the results to.
+         * @param cmp The {@link Comparator} to use to compare
+         *                {@link String}s.
+         * @param cs The {@link Charset} to be used for the byte to
+         *                character conversion.
+         * @param usegzip assumes we used gzip compression for temporary files
+         * @return The number of lines sorted.
+         * @throws IOException generic IO exception
+         * @since v0.1.4
+         */
+        public static long mergeSortedFiles(List<File> files, BufferedWriter fbw,
+                final Comparator<String> cmp, Charset cs, boolean distinct,
+                boolean usegzip) throws IOException {
+                ArrayList<IOStringStack> bfbs = new ArrayList<>();
+                for (File f : files) {
+                        final int BUFFERSIZE = 2048;
+                        if (f.length() == 0) {
+                                continue;
+                        }
+                        InputStream in = new FileInputStream(f);
+                        BufferedReader br;
+                        if (usegzip) {
+                                br = new BufferedReader(
+                                        new InputStreamReader(
+                                                new GZIPInputStream(in,
+                                                        BUFFERSIZE), cs));
+                        } else {
+                                br = new BufferedReader(new InputStreamReader(
+                                        in, cs));
+                        }
+
+                        BinaryFileBuffer bfb = new BinaryFileBuffer(br);
+                        bfbs.add(bfb);
+                }
+                long rowcounter = mergeSortedFiles(fbw, cmp, distinct, bfbs);
+                for (File f : files) {
+                        f.delete();
+                }
+                return rowcounter;
+        }
+
         /**
          * This sorts a file (input) to an output file (output) using default
          * parameters
@@ -447,7 +495,7 @@ public class ExternalSort {
          */
         public static void sort(final File input, final File output, final Comparator<String> cmp)
                 throws IOException {
-                ExternalSort.mergeSortedFiles(ExternalSort.sortInBatch(input),
+                ExternalSort.mergeSortedFiles(ExternalSort.sortInBatch(input, cmp),
                         output, cmp);
         }
 
@@ -876,41 +924,3 @@ public class ExternalSort {
         public static final int DEFAULTMAXTEMPFILES = 1024;
 
 }
-
-/**
- * This is essentially a thin wrapper on top of a BufferedReader... which keeps
- * the last line in memory.
- *
- */
-final class BinaryFileBuffer {
-        public BinaryFileBuffer(BufferedReader r) throws IOException {
-                this.fbr = r;
-                reload();
-        }
-        public void close() throws IOException {
-                this.fbr.close();
-        }
-
-        public boolean empty() {
-                return this.cache == null;
-        }
-
-        public String peek() {
-                return this.cache;
-        }
-
-        public String pop() throws IOException {
-                String answer = peek().toString();// make a copy
-                reload();
-                return answer;
-        }
-
-        private void reload() throws IOException {
-                this.cache = this.fbr.readLine();
-        }
-
-        public BufferedReader fbr;
-
-        private String cache;
-
-}
