@@ -4,24 +4,17 @@ import static org.junit.Assert.assertArrayEquals;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertNotNull;
 import static org.junit.Assert.assertTrue;
+import static org.junit.Assert.assertFalse;
 
-import java.io.BufferedReader;
-import java.io.File;
-import java.io.FileInputStream;
-import java.io.FileOutputStream;
-import java.io.FileReader;
-import java.io.IOException;
+import java.io.*;
 import java.nio.channels.FileChannel;
 import java.nio.charset.Charset;
 import java.nio.charset.StandardCharsets;
 import java.nio.file.Files;
 import java.nio.file.Path;
 import java.nio.file.StandardOpenOption;
-import java.util.ArrayList;
-import java.util.Arrays;
-import java.util.Comparator;
-import java.util.List;
-import java.util.Scanner;
+import java.util.*;
+import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.stream.Collectors;
 import java.util.stream.IntStream;
 
@@ -458,4 +451,79 @@ public class ExternalSortTest {
         return path;
     }
 
+    /**
+     * Sort with a custom comparator.
+     * @throws IOException
+     */
+    @Test
+    public void sortWithCustomComparator() throws IOException {
+        Random rand = new Random();
+        final Path path = Files.createTempFile("TestCsvWithLongIds", ".csv");
+        final Path pathSorted = Files.createTempFile("TestCsvWithLongIdsSorted", ".csv");
+        Set<Long> sortedIds = new TreeSet<>();
+        try (FileWriter fw = new FileWriter(path.toFile());
+             BufferedWriter bw = new BufferedWriter(fw)) {
+            for (int i = 0; i < 1000; ++i) {
+                long nextLong = rand.nextLong();
+                sortedIds.add(nextLong);
+                bw.write(String.format("%d,%s\n", nextLong, UUID.randomUUID().toString()));
+            }
+        }
+        AtomicBoolean wasCalled = new AtomicBoolean(false);
+        ExternalSort.sort(path.toFile(), pathSorted.toFile(), (lhs, rhs) -> {
+            Long lhsLong = lhs.indexOf(',') == -1 ? Long.MAX_VALUE : Long.parseLong(lhs.split(",")[0]);
+            Long rhsLong = rhs.indexOf(',') == -1 ? Long.MAX_VALUE : Long.parseLong(rhs.split(",")[0]);
+            wasCalled.set(true);
+            return lhsLong.compareTo(rhsLong);
+        });
+        assertTrue("The custom comparator was not called!", wasCalled.get());
+        Iterator<Long> idIter = sortedIds.iterator();
+        try (FileReader fr = new FileReader(pathSorted.toFile());
+             BufferedReader bw = new BufferedReader(fr)) {
+            String nextLine = bw.readLine();
+            Long lhsLong = nextLine.indexOf(',') == -1 ? Long.MAX_VALUE : Long.parseLong(nextLine.split(",")[0]);
+            Long nextId = idIter.next();
+            assertEquals(lhsLong, nextId);
+        }
+    }
+
+    @Test
+    public void lowMaxMemory() throws IOException {
+        String unsortedContent =
+                "Val1,Data2,Data3,Data4\r\n" +
+                "Val2,Data2,Data4,Data5\r\n" +
+                "Val1,Data2,Data3,Data5\r\n" +
+                "Val2,Data2,Data6,Data7\r\n";
+        InputStream bis = new ByteArrayInputStream(unsortedContent.getBytes(StandardCharsets.UTF_8));
+        File tmpDirectory = Files.createTempDirectory("sort").toFile();
+        tmpDirectory.deleteOnExit();
+
+        BufferedReader inputReader = new BufferedReader(new InputStreamReader(bis, StandardCharsets.UTF_8));
+        List<File> tmpSortedFiles = ExternalSort.sortInBatch(
+                inputReader,
+                unsortedContent.length(),
+                ExternalSort.defaultcomparator,
+                Integer.MAX_VALUE,  // use an unlimited number of temp files
+                100,                // max memory
+                StandardCharsets.UTF_8,
+                tmpDirectory,
+                false,              // no distinct
+                0,                  // no header lines to skip
+                false,              // don't use gzip
+                true);              // parallel
+        File tmpOutputFile = File.createTempFile("merged", "", tmpDirectory);
+        tmpOutputFile.deleteOnExit();
+        BufferedWriter outputWriter = new BufferedWriter(new OutputStreamWriter(new FileOutputStream((tmpOutputFile))));
+        ExternalSort.mergeSortedFiles(
+                tmpSortedFiles,
+                outputWriter,
+                ExternalSort.defaultcomparator,
+                StandardCharsets.UTF_8,
+                false,              // no distinct
+                false);             // don't use gzip
+
+        for (File tmpSortedFile: tmpSortedFiles) {
+            assertFalse(tmpSortedFile.exists());
+        }
+    }
 }
